# -*- coding: utf-8 -*-
"""
Written by Evan Lalopoulos <evan.lalopoulos.2017@my.bristol.ac.uk>
University of Bristol, May 2018
Copyright (C) - All Rights Reserved
"""

corpus = [
    'The quick brown fox jumped over the lazy dog.',
    'My dog also likes eating sausage.'
]

corpus_ner = [
    "Hillary Clinton agrees with John McCain \"by voting to give George Bush the benefit of the doubt on Iran.\"",
    "My dog also likes eating sausage."
]